@@ -17,7 +17,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_BADCREDENTIALS
 );
 
-our $VERSION = '2.0.16';
+our $VERSION = '2.19.0';
 
 extends 'Lemonldap::NG::Portal::Main::SecondFactor';
 with 'Lemonldap::NG::Portal::Lib::WebAuthn';
@@ -53,6 +53,9 @@ sub run {
 
     $self->ott->updateToken( $token, _webauthn_request => $request );
 
+    $self->logger->debug(
+        "WebAuthn authentication parameters " . to_json($request) );
+
     # Prepare form
     my ( $checkLogins, $stayConnected ) = $self->getFormParams($req);
     my $tmp = $self->p->sendHtml(
