@@ -9,8 +9,9 @@ use strict;
 use Mouse;
 use MIME::Base64;
 use Lemonldap::NG::Common::FormEncode;
+use URI;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.16.3';
 
 extends 'Lemonldap::NG::Common::PSGI';
 
@@ -163,7 +164,10 @@ sub handler {
         );
 
         # Redirect
-        return [ 302, [ Location => $urldc, $req->spliceHdrs ], [] ];
+        return [
+            302, [ Location => URI->new($urldc)->as_string, $req->spliceHdrs ],
+            []
+        ];
 
     }
 
