@@ -22,7 +22,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_MAILCONFIRMATION_ALREADY_SENT
 );
 
-our $VERSION = '2.16.1';
+our $VERSION = '2.19.0';
 
 extends qw(
   Lemonldap::NG::Portal::Lib::SMTP
@@ -35,20 +35,6 @@ extends qw(
 # Sub module (Demo, LDAP,...)
 has registerModule => ( is => 'rw' );
 
-# Register URL to set in mail
-has registerUrl => (
-    is      => 'ro',
-    lazy    => 1,
-    default => sub {
-        my $self = $_[0];
-        my $p    = $self->conf->{portal};
-        $p =~ s#/*$##;
-        return $self->conf->{registerUrl}
-          ? $self->conf->{registerUrl}
-          : "$p/register";
-    }
-);
-
 # Mail timeout token generator
 has mailott => (
     is      => 'rw',
@@ -278,13 +264,14 @@ sub _register {
         # Build confirmation url
         my $req_url = $req->data->{_url};
         my $skin    = $self->p->getSkin($req);
-        my $url =
-          $self->registerUrl . '?'
-          . build_urlencoded(
-            register_token => $req->{id},
-            skin           => $skin,
-            ( $req_url ? ( url => $req_url ) : () ),
-          );
+        my $url     = $self->p->buildUrl(
+            ( $self->conf->{registerUrl} || ( $req->portal, "register" ) ),
+            {
+                register_token => $req->{id},
+                skin           => $skin,
+                ( $req_url ? ( url => $req_url ) : () ),
+            }
+        );
 
         # Build mail content
         my $tr      = $self->translate($req);
@@ -361,15 +348,15 @@ sub _register {
     }
 
     # Build portal url
-    my $url = $self->conf->{portal};
-    $url =~ s#/*$##;
     my $req_url = $req->data->{_url};
     my $skin    = $self->p->getSkin($req);
-    $url .= '/?'
-      . build_urlencoded(
-        skin => $skin,
-        ( $req_url ? ( url => $req_url ) : () ),
-      );
+    my $url     = $self->p->buildUrl(
+        $req->portal,
+        {
+            skin => $skin,
+            ( $req_url ? ( url => $req_url ) : () ),
+        }
+    );
 
     # Build mail content
     my $tr      = $self->translate($req);
